package jp.sourceforge.pdt_tools.iwbhistory.preferences;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.runtime.Status;
import org.eclipse.jface.preference.ListEditor;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.internal.browser.BrowserViewer;
import org.eclipse.ui.internal.browser.WebBrowserPreference;
import org.eclipse.ui.internal.browser.WebBrowserUIPlugin;
import org.eclipse.ui.internal.browser.WebBrowserView;

@SuppressWarnings("restriction")
public class HistoryEditor extends ListEditor {

	private List<String> iwbHistory;

	public HistoryEditor(String name, String labelText, Composite parent) {
		super(name, labelText, parent);
		iwbHistory = new ArrayList<String>();
	}

	@Override
	protected String createList(String[] items) {
		return null;
	}

	@Override
	protected String getNewInputObject() {
		return null;
	}

	@Override
	protected String[] parseString(String stringList) {
		return null;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	@Override
	public void load() {
		org.eclipse.swt.widgets.List list = getList();
		if (list != null && !list.isDisposed()) {
			List history = WebBrowserPreference.getInternalWebBrowserHistory();
			list.setItems((String[]) history.toArray(new String[history.size()]));
			iwbHistory.addAll(history);
		}
	}

	@Override
	public void loadDefault() {
		org.eclipse.swt.widgets.List list = getList();
		if (list != null && !list.isDisposed()) {
			list.setItems(new String[0]);
		}
	}

	@Override
	public void store() {
		org.eclipse.swt.widgets.List list = getList();
		if (list != null && !list.isDisposed()) {
			String[] items = list.getItems();
			if (isModified(items)) {
				WebBrowserPreference.setInternalWebBrowserHistory(Arrays
						.asList(items));
				resetBrowser(items);
			}
		}
	}

	@Override
	public Composite getButtonBoxControl(Composite parent) {
		Composite composite = super.getButtonBoxControl(parent);
		getAddButton().dispose();
		return composite;
	}

	private boolean isModified(String[] list) {
		if (list.length == iwbHistory.size()) {
			for (int i = 0; i < list.length; i++) {
				if (!list[i].equals(iwbHistory.get(i))) {
					return true;
				}
			}
			return false;
		}
		return true;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void resetBrowser(String[] items) {
		try {
			IViewPart viewPart = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow().getActivePage()
					.showView(WebBrowserView.WEB_BROWSER_VIEW_ID);
			if (viewPart instanceof WebBrowserView) {
				Field viewer = viewPart.getClass().getDeclaredField("viewer"); //$NON-NLS-1$
				viewer.setAccessible(true);
				Object browserView = viewer.get(viewPart);
				if (browserView instanceof BrowserViewer) {
					Field history = browserView.getClass().getDeclaredField(
							"history"); //$NON-NLS-1$
					history.setAccessible(true);
					Object historyList = history.get(browserView);
					if (historyList instanceof List) {
						((List) historyList).clear();
						((List) historyList).addAll(Arrays.asList(items));
						Method updateHistory = browserView.getClass()
								.getDeclaredMethod("updateHistory"); //$NON-NLS-1$
						updateHistory.setAccessible(true);
						updateHistory.invoke(browserView);
					}
				}
			}
		} catch (Exception e) {
			log(e);
		}
	}

	final static String PLUGIN_ID = "jp.sourceforge.pdt_tools.iwbhistory"; //$NON-NLS-1$

	private void log(Throwable e) {
		WebBrowserUIPlugin
				.getInstance()
				.getLog()
				.log(new Status(Status.ERROR, PLUGIN_ID, e
						.getLocalizedMessage()));
	}

}
